Cartouche.Erc20.Call (Cartouche v0.3.0)

Copy Markdown View Source

Module to call operations and receive return value, without sending a transaction.

API Functions

FunctionArityDescriptionParam Kinds
transfer4Call ERC-20 transfer(address,uint256) without sending a transaction.token: value, destination: value, amount_wei: value, call_opts: value
balance_of3Call ERC-20 balanceOf(address) and decode the token base-unit balance.token: value, address: value, call_opts: value

Summary

Functions

Calls the balanceOf operation, returning the result of the Ethereum function call.

Calls the transfer operation, returning the result of the Ethereum function call.

Functions

balance_of(token, address, call_opts \\ [])

@spec balance_of(
  Cartouche.contract(),
  Cartouche.address(),
  Cartouche.Erc20.call_opts()
) ::
  {:ok, number()} | {:error, term()}

Calls the balanceOf operation, returning the result of the Ethereum function call.

Examples

iex> Cartouche.Erc20.Call.balance_of(<<0xCC>>, <<0xDD>>)
{:ok, <<>>}

transfer(token, destination, amount_wei, call_opts \\ [])

@spec transfer(
  Cartouche.contract(),
  Cartouche.address(),
  non_neg_integer(),
  Cartouche.Erc20.call_opts()
) :: {:ok, binary()} | {:error, term()}

Calls the transfer operation, returning the result of the Ethereum function call.

Examples

iex> Cartouche.Erc20.Call.transfer(<<0xCC>>, <<0xDD>>, 100_000)
{:ok, <<>>}